#ifndef BASEDEDONNEES_H
#define BASEDEDONNEES_H

#include <QObject>
#include <QtSql/QtSql>
#include <QSqlDatabase>
#include <QMutex>

#ifdef DEBUG_BASEDEDONNEES
    //#define DEBUG_BASEDEDONNEES_1
#endif

/**
 * @brief Classe de gestion de base de données SQLite
 *
 * @details Classe singleton + ressource critique protégée par un mutex
 *
 * @warning Il faut ajouter QT += sql au fichier .pro
 *
 * @class BaseDeDonnees basededonnees.h "basededonnees.h"
 *
 * @author Thierry Vaira <tvaira@free.fr>
 */
class BaseDeDonnees : public QObject
{
    Q_OBJECT
    public:
        /**
         * @brief Retourne l'unique instance (membre statique)
         *
         * @fn getInstance
         * @return BaseDeDonnees
         */
        static BaseDeDonnees* getInstance();
        /**
         * @brief Libère l'instance obtenue
         *
         * @fn detruireInstance
         */
        static void detruireInstance();

        /**
         * @brief Se connecte à une base de données
         *
         * @fn connecter
         * @param nomReferentiel QString nom du fichier représentant la base de données (par défaut referentiel-sai.sqlite)
         * @return bool vrai si la connexion a réussi sinon faux
         */
        bool connecter(QString nomReferentiel = "referentiel-sai.sqlite");
        /**
         * @brief Exécute une requête SQL UPDATE, INSERT ou DELETE
         *
         * @fn executer
         * @param requete QString une requête SQL UPDATE, INSERT ou DELETE
         * @return bool vrai si la requête a été exécutée avec succès sinon faux
         */
        bool executer(QString requete); /* pour : UPDATE, INSERT et DELETE */
        /* uniquement pour : SELECT (enregistrement -> champ) */
        /**
         * @brief Exécute une requête SQL SELECT pour récupérer un seul champ de plusieurs enregistrements : le champ des différents enregistrements est stocké dans un QVector de QString
         *
         * @fn recuperer(QString, QVector<QString> &)
         * @param requete QString
         * @param donnees QVector<QString>
         * @return bool vrai si la requête a été exécutée avec succès sinon faux
         */
        bool recuperer(QString requete, QVector<QString> &donnees); // 1..* -> 1
        /**
         * @brief Exécute une requête SQL SELECT pour récupérer plusieurs champs de plusieurs enregistrements : les différents champs des différents enregistrements sont stockés dans un QVector de QStringList
         *
         * @fn recuperer(QString, QVector<QStringList> &)
         * @param requete QString
         * @param donnees QVector<QStringList>
         * @return bool vrai si la requête a été exécutée avec succès sinon faux
         */
        bool recuperer(QString requete, QVector<QStringList> &donnees); // 1..* -> 1..*
        /**
         * @brief Exécute une requête SQL SELECT pour récupérer un seul champ d’un seul enregistrement : le champ est stocké dans un QString
         *
         * @fn recuperer(QString, QString &)
         * @param requete QString
         * @param donnees QString
         * @return bool vrai si la requête a été exécutée avec succès sinon faux
         */
        bool recuperer(QString requete, QString &donnees); // 1 -> 1
        /**
         * @brief Exécute une requête SQL SELECT pour récupérer plusieurs champs d’un seul enregistrement : les différents champs sont stockés dans un QStringList
         *
         * @fn recuperer(QString, QStringList &)
         * @param requete QString
         * @param donnees QStringList
         * @return bool vrai si la requête a été exécutée avec succès sinon faux
         */
        bool recuperer(QString requete, QStringList &donnees); // 1 -> 1..*

    private:
        /**
         * @brief Constructeur placé en privé pour interdire la création d'une instance de l'extérieur de la classe
         *
         * @fn BaseDeDonnees
         */
        BaseDeDonnees();
        /**
         * @brief Destructeur
         *
         * @fn ~BaseDeDonnees
         */
        ~BaseDeDonnees();
        static BaseDeDonnees*  baseDeDonnees; /**< pointeur sur l'instance de la classe (membre statique) */
        static int             nbAcces; /**< nombre d'objets en cours d'utilisation de l'instance de la classe */
        QSqlDatabase           db; /**< un objet QSqlDatabase */
        QMutex                 mutex; /**< un mutex pour protéger des accès concurrents à la base de données */
};

#endif // BASEDEDONNEES_H
