#include "panneau.h"
#include "port.h"
#include "siv.h"
#include <iostream>


using namespace std;

/*
 * Rôle : instancie un objet Port
 *
 * Paramètre(s) en entrée : portPanneau de type QString contenant le chemin vers le fichier de périphérique [valeur par défaut "/dev/panneau"]
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune
 */

Panneau::Panneau(QString portPanneau)
{
    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << "-> port : " << portPanneau;
    #endif
    
    #ifndef SIMULATION_PANNEAU
    port = new Port(portPanneau);
    #else
    Q_UNUSED(port)
    #endif
}

/*
 * Rôle : libère l'objet Port
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune
 */

Panneau::~Panneau()
{
    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO;
    #endif
    #ifndef SIMULATION_PANNEAU
    delete port;
    #endif
}

/*
 * Rôle de la méthode : Cette méthode permet d'afficher le texte contenu dans la variable "message"
 *
 * Contrainte(s) : le panneau ne peut afficher que des textes de 10 caractères max
 * 
 * Paramètre(s) en entrée : 
 *  - message : de type QString contenant la chaîne de caractère à afficher [valeur par défaut : aucune]
 *  - position : de type int (entier) contenant la position sur l'afficheur du premier caractère du message [valeur par défaut 11]
 *  - duree : de type int (entier) contenant la durée de maintien de l'affichage exprimée en secondes [valeur par défaut 1s]
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

void Panneau::afficher(QString message, int duree, int position)
{
    char trame[MAX_TRAME] = {FIN};
        
    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << "-> message : " << message << "position : " << position;
    #endif

    // on fabrique la trame
    fabriquerTrame(&trame[0], message, position);
    
    // on envoie la trame
    envoyerTrame(trame);

    sleep(duree);
}

/*
 * Rôle de la méthode : Cette méthode fait défiler le texte de la droite vers la gauche
 *
 * Limite(s) : texte limité à 12 caractères
 * 
 * Paramètre(s) en entrée :
 *  - message : de type QString contenant la chaîne de caractère à afficher [valeur par défaut : aucune]
 *  - position : de type int (entier) contenant la position sur l'afficheur du premier caractère du message [valeur par défaut 11]
 *  - vitesse : de type int (entier) contenant la vitesse de défilement exprimée en millisecondes [valeur par défaut 250 ms]
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (void)
 *
 */

void Panneau::defiler(QString message, int vitesse, int position)
{
    char trame[MAX_TRAME] = {FIN};
    QString texte;

    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << "-> message : " << message << "position : " << position;
    #endif
    for (int i = 0 ; i < message.length() ; i++)
    {
        texte = message.mid(i, MAX_TEXTE);
#ifdef DEBUG_PANNEAU
//qDebug() << Q_FUNC_INFO << "-> texte : " << texte;
#endif
        fabriquerTrame(&trame[0], texte, position);
        envoyerTrame(trame);

        usleep(vitesse*1000);
    }
}

/*
 * Rôle de la méthode : Cette méthode permet d'effacer le Panneau pour un prochain affichage
 *
 * Contrainte(s) :
 *
 * Paramètre(s) en entrée :
 *  - duree : de type int (entier) contenant la durée de maintien de l'affichage exprimée en secondes [valeur par défaut : aucune]
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

void Panneau::effacer(int duree)
{
    char trame[MAX_TRAME] = {FIN};
    QString message = "";

    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO;
    #endif

    // on fabrique la trame
    fabriquerTrame(&trame[0], message);

    // on envoie la trame
    envoyerTrame(trame);

    sleep(duree);
}

/*
 * Rôle de la méthode : Cette méthode permet d'envoyer la trame
 *
 * Visibilité : private
 *
 * Paramètre(s) en entrée : trame[MAX_TRAME]
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : vrai si la trame a été envoyée entièrement sinon faux
 */

bool Panneau::envoyerTrame(char trame[MAX_TRAME])
{
    int retour = 0;
    bool etat = false;

    #ifndef SIMULATION_PANNEAU
    retour = port->transmettre(trame);
    #else
    Q_UNUSED(trame)
    retour = 1; // ok
    #endif

    // si l'envoi de la trame echoue ?
    if (retour < 1)
    {
        #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << "-> erreur emission trame !";
        #endif
        etat = false;
    }
    else
    {
        if(retour == (int)strlen(trame))
            etat = true;
        else
            etat = false;
    }

    return etat;
}

/*
 * Rôle de la méthode : Fabrique la trame en fonction de la documentation
 *
 * Visibilité : private
 *
 * Paramètre(s) en entrée : trame, message et position.
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : un entier pour indiquer que la trame a été fabriquée
 */

int Panneau::fabriquerTrame(char *trame, QString message, int position)
{
    char trameposition[4]= {FIN};
    QString messageNettoye;

    // réinitialise la trame à 0
    memset(&trame[0], 0, MAX_TRAME);

    trame[0] = SOT;
    trame[1] = NUM;
    trame[2] = NUM;
    trame[3] = CMD;
    trame[4] = AUX;
    trame[5] = F;
    trame[6] = F;
    trame[7] = AUX;
    trame[8] = Z;
    trame[9] = Z;
    trame[10] = STX;
    trame[11] = CHAN;
    trame[12] = LINE;

    sprintf(trameposition,"%02d",position);

    trame[13] = trameposition[0];
    trame[14] = trameposition[1];

    int i = 15;
    int j = 0;    

    messageNettoye = nettoyer(message);

    // 17 octets de protocole (taille fixe)
    for (j = 0 ; j < messageNettoye.length() && j < (MAX_TRAME - 17) ; j++)
    {
        trame[i++] = messageNettoye.at(j).toAscii();
    }

    trame[i] = ETX;
    trame[i+1] = EOT;

    return 1;
}

QString Panneau::nettoyer(QString message)
{
    QString messageNettoye;

    messageNettoye = message.replace(QString::fromUtf8("ç"), "c");
    messageNettoye = message.replace(QString::fromUtf8("â"), "a");
    messageNettoye = message.replace(QString::fromUtf8("ê"), "e");
    messageNettoye = message.replace(QString::fromUtf8("î"), "i");
    messageNettoye = message.replace(QString::fromUtf8("ô"), "o");
    messageNettoye = message.replace(QString::fromUtf8("û"), "u");
    messageNettoye = message.replace(QString::fromUtf8("à"), "a");
    messageNettoye = message.replace(QString::fromUtf8("è"), "e");
    messageNettoye = message.replace(QString::fromUtf8("é"), "e");
    messageNettoye = message.replace(QString::fromUtf8("ï"), "i");
    messageNettoye = message.replace(QString::fromUtf8("ë"), "e");

    return messageNettoye;
}
