#ifndef _PANNEAU_H
#define _PANNEAU_H

#include <QtCore>

#include <cstdio>
#include <cstdlib>
#include <termios.h>
#include <unistd.h>
#include <sys/fcntl.h>
#include <string.h>
#include <iostream>

using namespace std;

//#define SIMULATION_PANNEAU

// Paramètres par défaut
/// Nom et chemin vers le fichier de périphérique du panneau
#define PORT_DEFAUT     "/dev/panneau"

/// Longueur maximale d'une trame
#define MAX_TRAME       64
/// Longueur maximale d'affichage d'un texte
#define MAX_TEXTE       10

// Protocole
/// Début de trame (Start Of Transmission)
#define SOT             0x01
/// Numéro de panneau
#define NUM             0x30
/// Commande d'affichage d'un texte
#define CMD             0x06
/// Fonction auxiliare
#define AUX             0x0F
/// Le caractère 'F'
#define F               0x46
/// Le caractère Zéro
#define Z               0x30
/// Début de texte (Start of TeXte)
#define STX             0x02
/// Numéro de canal de l'afficheur
#define CHAN            0x31
/// Numéro de ligne de l'afficheur
#define LINE            0x32
/// Position par défaut pour le premier caractère du texte
#define POS_DEFAUT      11
/// Fin de texte (End of TeXte)
#define ETX             0x03
/// Fin de trame (Enf Of Transmission)
#define EOT             0x04
/// Fin de chaîne (caractère NUL)
#define FIN             0x00 // fin de chaîne

class Port; // communication port série

/**
 * @brief Classe qui gère l'affichage d'informations sur le panneau
 *
 * @class Panneau panneau.h "panneau.h"
 *
 * @author Gabriel Destrée <gabriel.destree@gmail.com>
 */
class Panneau : public QObject
{
  Q_OBJECT

  private:
    Port *port; /**< Pointeur sur un objet Port */

    /**
     * @brief Envoie la trame sur le panneau
     *
     * @fn envoyerTrame
     * @param trame[] un tableau de MAX_TRAME caractères
     * @return bool vrai si la trame a été envoyée entièrement sinon faux
     */
    bool envoyerTrame(char trame[MAX_TRAME]);
    /**
     * @brief Fabrique la trame en fonction du protocole
     *
     * @fn fabriquerTrame
     * @param trame pointeur sur un tableau de MAX_TRAME caractères
     * @param message QString contenant le message à afficher
     * @param position entier contenant la position sur l'afficheur du premier caractère du message (valeur par défaut 11)
     * @return int entier indiquant si la trame a été fabriquée
     */
    int  fabriquerTrame(char *trame, QString message, int position=POS_DEFAUT);

    QString nettoyer(QString message);

  public:
    /**
     * @brief Constructeur de la classe Panneau
     *
     * @fn Panneau
     * @param portPanneau QString contenant le chemin vers le fichier de périphérique (valeur par défaut "/dev/panneau")
     */
    Panneau(QString portPanneau=PORT_DEFAUT);
    /**
     * @brief Destructeur de la classe Panneau
     *
     * @fn ~Panneau
     */
    ~Panneau();

    /**
     * @brief Affichage fixe d'un message sur le Panneau (< 10 caractères)
     *
     * @fn afficher
     * @param message QString contenant la chaîne de caractère à afficher (valeur par défaut : aucune)
     * @param duree entier contenant la durée de maintien de l'affichage exprimée en secondes (valeur par défaut : aucune)
     * @param position entier contenant la position sur l'afficheur du premier caractère du message (valeur par défaut 11)
     */
    void afficher(QString message, int duree, int position=POS_DEFAUT);
    /**
     * @brief Défilement de texte sur le Panneau (> 10 caractères)
     *
     * @fn defiler
     * @param message QString contenant la chaîne de caractère à afficher (valeur par défaut : aucune)
     * @param vitesse entier contenant la vitesse de défilement exprimée en millisecondes (valeur par défaut : aucune)
     * @param position entier contenant la position sur l'afficheur du premier caractère du message (valeur par défaut 11)
     */
    void defiler(QString message, int vitesse, int position=POS_DEFAUT);
    /**
     * @brief Permet d'effacer le Panneau pour un prochain affichage
     *
     * @fn effacer
     * @param duree entier contenant la durée de maintien de l'affichage exprimée en secondes (valeur par défaut : aucune)
     */
    void effacer(int duree);
};

#endif
