#ifndef PORT_H
#define PORT_H

#include <QObject>
#include <QList>

#include "qextserialenumerator.h"
#include "qextserialport.h"

#define BUFFERSIZE 1024 

/**
 * @brief Classe de gestion d'un port série
 *
 * @class Port port.h "port.h"
 *
 * @author Thierry Vaira <tvaira@free.fr>
 */
class Port : public QObject
{
Q_OBJECT
public:

    /**
     * @brief Constructeur par défaut
     *
     * @fn Port
     */
    Port();
    /**
     * @brief Constructeur
     *
     * @fn Port(const QString &)
     * @param portName QString indiquant le nom et chemin vers le fichier de périphérique
     */
   Port(const QString &portName);
   /**
    * @brief Destructeur
    *
    * @fn ~Port
    */
   ~Port();
   
   /**
    * @brief Ouvre et configure un port série
    *
    * @fn ouvrir(const QString &, int)
    * @param portName QString indiquant le nom et chemin vers le fichier de périphérique
    * @param debit entier précisant le débit (9600 bits/s par défaut)
    * @return bool vrai si le port est ouvert sinon faux
    */
   bool ouvrir(const QString &portName, int debit = 9600);
   /**
    * @brief Transmet un message sur le port série
    *
    * @fn transmettre(const QString &)
    * @param message const QString & le message à transmettre
    * @return int nombre de caractères transmis
    */
   int  transmettre(const QString &message);
   /**
    * @brief Transmet un message sur le port série
    *
    * @fn transmettre(char *)
    * @param message char * pointeur sur le message à transmettre
    * @return int nombre de caractères transmis
    */
   int  transmettre(char *message);
   /**
    * @brief Réceptionne un message sur le port série
    *
    * @fn recevoir(QString &)
    * @param message le message reçu
    * @return int nombre d'octets reçus
    */
   int  recevoir(QString &message);
   /**
    * @brief Lit un caractère sur le port série
    *
    * @fn lire()
    * @return char le caractère reçu
    */
   char lire();
   /**
    * @brief Lit n caractères sur le port série
    *
    * @fn lire(char *, int)
    * @param buffer pointeur sur un tableau de caractères
    * @param n nombre de caractères à lire
    * @return int nombre de caractères lus
    */
   int  lire(char *buffer, int n);
   /**
    * @brief Retourne le nombre d'octets disponible pour une lecture
    *
    * @fn getNbOctetsDisponibles
    * @return long nombre d'octets disponible pour une lecture
    */
   long getNbOctetsDisponibles();
   /**
    * @brief Retourne l'état du port série
    *
    * @fn estOuvert
    * @return bool vrai si le port est ouvert sinon faux
    */
   bool estOuvert();
   
private:

   QextSerialPort *port; /**< pointeur sur un objet QextSerialPort */

private slots:

   /**
    * @brief Slot non utilisé
    *
    * @fn onReadyRead
    */
   void onReadyRead();
};

#endif // PORT_H
