#ifndef _QTP_H
#define _QTP_H

#include <QtCore> 
#include <stdio.h> 
#include <errno.h> 
#include <termios.h> 
#include <stdlib.h>
#include <string.h> 
#include <unistd.h>
#include <sys/ioctl.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/signal.h>
#include <fcntl.h> 

//codes etat
#define QTP_READY		0x06
#define QTP_NOTREADY	0x15

//limites EEPROM
#define MAX_MESS_256	9
#define MAX_MESS_512	22
#define MAX_MESS_1024	47
#define MAX_MESS_2048	99
#define MAX_LENGTH		20

//codes cmds
#define NB_CMDS				21
#define LG_CMD				6
#define ABS					0
#define CLEAR_ENDOFLINE		1
#define CLEAR_ENDOFPAGE		2
#define CURSOR_OFF			3
#define CURSOR_ON			4
#define BLINK				5
#define REQUEST_READY		6
#define TEST_WRITE			7
#define TEST_READ			8
#define KEY_RECONFIG		9
#define KEYCLICK_ON			10
#define KEYCLICK_OFF		11
#define KEYCLICK_ON_SAVE	12
#define KEYCLICK_OFF_SAVE	13
#define READ_VERSION		14
#define READ_NUMBER			15
#define MESSAGE_STORING		16
#define MESSAGE_READING		17
#define MESSAGE_DISPLAY		18
#define MESSAGE_SCROLL		19
#define INPUTS_CONFIG		20
#define INPUTS_READING		21

//codes QTP
#define CURSOR_LEFT     0x15
#define CURSOR_RIGHT	0x06
#define CURSOR_DOWN     0x0A
#define CURSOR_UP		0x1A
#define CURSOR_HOME     0x01
#define CR				0x0D
#define LF				0x0A 
#define CRLF			0x1D
#define BS				0x08
#define CLEAR_PAGE      0x0C
#define CLEAR_LINE      0x19
#define BELL			0x07
#define BEEP			0x07
#define ESC				0x1B
#define ACK				0x06
#define NACK			0x15
#define TEST			0x55

//divers
#define ERROR	-1
#define VRAI	1
#define FAUX	0

#define LG_LIGNE	20  
#define NB_LIGNES	4

#define QTP_ECHO	1
#define QTP_NOECHO	0

//codes clavier
#define DIESE		0x0D
#define ETOILE		0x1B

//macro: récupérer le nombre d'octets d'une commande
#define NB_CHAR(x)	(cmds[x][LG_CMD-1])


/**
 * @brief Classe de bas niveau qui gère le terminal QTP
 *
 * @class QTP qtp.h "qtp.h"
 *
 * @author Thierry Vaira <tvaira@free.fr>
 */
class QTP
{
public:
    /**
     * @brief Constructeur
     *
     * @fn QTP(QString port = "/dev/qtp")
     * @param port nom et chemin vers le fichier de périphérique
     */
    QTP(QString port = "/dev/qtp");
    /**
     * @brief Destructeur
     *
     * @fn ~QTP
     */
    ~QTP();
    
    /**
     * @brief Affiche un message de nbchar caractères
     *
     * @fn afficher
     * @param message
     * @param nbchar
     * @return int
     */
    int afficher(char* message, int nbchar);
    /**
     * @brief Lit le code d'une touche avec ous sans le mode echo
     *
     * @fn lireTouche
     * @param touche
     * @param nbchar
     * @param mode
     * @return char
     */
    char lireTouche(char *touche, int nbchar, int mode);
    
    /**
     * @brief Déplace le curseur à la position : ligne,colonne
     *
     * @fn gotoxy
     * @param ligne
     * @param colonne
     */
    void gotoxy(int ligne, int colonne);
    /**
     * @brief Efface la fin d'une ligne
     *
     * @fn clearEndofline
     */
    void clearEndofline();
    /**
     * @brief Efface la fin de la page
     *
     * @fn clearEndofpage
     */
    void clearEndofpage();
    /**
     * @brief Active le curseur
     *
     * @fn cursorOn
     */
    void cursorOn();
    /**
     * @brief Désactive le curseur
     *
     * @fn cursorOff
     */
    void cursorOff();
    /**
     * @brief Fait clignoter le curseur
     *
     * @fn cursorBlink
     */
    void cursorBlink();

protected:	
    /**
     * @brief Émet nbchar caractères sur le port série
     *
     * @fn send
     * @param cmd
     * @param nbchar
     * @return int
     */
    int send(char *cmd, int nbchar);
    /**
     * @brief Reçoit nbcahr caractères du port série
     *
     * @fn recv
     * @param mess
     * @param nbchar
     * @return int
     */
    int recv(char *mess, int nbchar);
    /**
     * @brief Émet une requête
     *
     * @fn requestReady
     * @return int
     */
    int requestReady();

private:
    int _qtpid; /**< descripteur de périphérique qtp */
};

#endif

