#ifndef TDIFFUSIONMESSAGES_H
#define TDIFFUSIONMESSAGES_H

#include <QtCore>
#include "panneau.h"

/**
 * @brief Classe qui gère la diffussion des messages informatifs dans le bus
 *
 * @class TDiffusionMessages tdiffusionmessages.h "tdiffusionmessages.h"
 *
 * @author Gabriel Destrée <gabriel.destree@gmail.com>
 */
class TDiffusionMessages : public QObject
{
    Q_OBJECT

private:
    Panneau panneau; /**< Objet panneau de type Panneau */
    int periode; /**< Un int correspondant à la période d'une diffusion complète */
    int duree; /**< Un int correspondant à la durée d'affichage d'un message */
    int transition; /**< Un int correspondant à la durée de maintien de l'effacement */
    int vitesse; /**< Un int correspondant à la vitesse de défilement */
    bool informationsVoyageursPresentes; /**< Un bool permettant de savoir si les informations voyageurs sont présentes ou non */
    QString numLigne; /**< Un QString contenant le numéro de ligne */
    QString destination; /**< Un QString contenant le nom de la destination */
    QString arretEnCours; /**< Un QString contenant le nom de l'arrêt en cours */
    QString prochainArret; /**< Un QString contenant le nom du prochain arrêt */
    bool fini; /**< Un bool permettant de savoir si le thread est terminé */
    QMutex mutex; /**< mutex pour la gestion des temporisations */
    QWaitCondition waitCondition; /**< pour la gestion des temporisations */
    /**
     * @brief Permet de faire des temporisations dans le thread
     *
     * @fn msleep
     * @param sleepMS
     */
    void msleep(unsigned long sleepMS)
    {
        waitCondition.wait(&mutex, sleepMS);
    }
    /**
     * @brief Annule une temporisation en cours
     *
     * @fn annuler
     */
    void annuler()
    {
        waitCondition.wakeAll();
    }

public:
/**
 * @brief Constructeur de la classe TDiffusionMessages
 *
 * @fn TDiffusionMessages
 */
    TDiffusionMessages();
    /**
     * @brief Destructeur de la classe TDiffusionMessages
     *
     * @fn ~TDiffusionMessages
     */
    ~TDiffusionMessages();

    // Prototypes méthodes

    /**
     * @brief Lecture du fichier .ini et de ses paramètres
     *
     * @fn lireParametres
     */
    void lireParametres();
    /**
     * @brief Méthode principale pour la diffusion des messages
     *
     * @fn diffuser
     */
    void diffuser();

    /**
     * @brief Méthode qui permet l'affichage de l'heure actuelle
     *
     * @fn afficherHeure
     * @param position du premier caractère du message sur le Panneau
     */
    void afficherHeure(int position=POS_DEFAUT+3);
    /**
     * @brief Méthode qui permet l'affichage de la date courante
     *
     * @fn afficherDate
     * @param position du premier caractère du message sur le Panneau
     */
    void afficherDate(int position=POS_DEFAUT);

    /**
     * @brief Méthode qui permet l'affichage du numéro de Ligne
     *
     * @fn afficherLigne
     * @param numLigne
     */
    void afficherLigne(QString numLigne);
    /**
     * @brief Méthode qui permet l'affichage de la destination du bus
     *
     * @fn afficherDestination
     * @param destination
     */
    void afficherDestination(QString destination);
    /**
     * @brief Méthode qui permet l'affichage de l'arrêt et du prochain arrêt du bus
     *
     * @fn afficherArret
     * @param nomArretEnCours
     * @param nomProchainArret
     */
    void afficherArret(QString nomArretEnCours, QString nomProchainArret);
    /**
     * @brief Méthode d'attente d'une prochaine diffusion
     *
     * @fn attendrePeriode
     */
    void attendrePeriode();

public slots:
    /**
     * @brief Le corps du thread
     *
     * @fn main
     */
    void main();
    /**
     * @brief Met fin à la boucle du thread
     *
     * @fn finir
     */
    void finir();
    /**
     * @brief Termine le thread
     *
     * @fn terminer
     */
    void terminer();

    /**
     * @brief Actualisation des Informations Voyageurs utiles aux afficheurs
     *
     * @fn actualiserInformationsVoyageur
     * @param numLigne
     * @param destination
     * @param arretEnCours
     * @param prochainArret
     */
    void actualiserInformationsVoyageur(QString numLigne, QString destination, QString arretEnCours, QString prochainArret, int numArret);

signals:

};

#endif // TDIFFUSIONMESSAGES_H
