#ifndef TGESTIONCONDUCTEUR_H
#define TGESTIONCONDUCTEUR_H

#include <QtCore>
#include "pupitreconducteur.h"
#include "basededonnees.h"

/// Choix du menu SERVICE
#define AUCUN           0
#define PRISE_SERVICE   1
#define FIN_SERVICE     2
#define DEMARRER_COURSE 3
#define AFFICHER_COURSE 4

/// Etat du bus : PAS_EN_COURSE pour l'attribut enCourse
#define PAS_EN_COURSE   0
/// Etat du bus : EN_COURSE pour l'attribut enCourse
#define EN_COURSE       1
/// Etat du bus : COURSE_TERMINEE pour l'attribut enCourse
#define COURSE_TERMINEE 2

#define PAS_EN_SERVICE 0
#define EN_SERVICE 1

typedef struct
{
    QString idItineraire;
    QString numLigne;
    QString destination;
    QString prochainArret;
    QString heureDepart;
    QString numArret;
    int     avanceRetard;
}
InformationsConducteur;

/**
 * @brief Classe qui gère la gestion du conducteur : Prise de service, prendre une courses, afficher l'itinéraire..
 *
 * @class TGestionConducteur tgestionconducteur.h "tgestionconducteur.h"
 *
 * @author Cyril Cambe <cambecyril@yahoo.fr>
 */
class TGestionConducteur : public QObject
{
    Q_OBJECT

private:
    PupitreConducteur pupitreConducteur; /**< Objet pupitreConducteur de type PupitreConducteur */
    InformationsConducteur informationsConducteur; /**< pour l'affichage pupitre */
    bool fini;     /**< Un bool pour savoir si la gestion est fini ou non */
    QString idVehicule; /**< QString permettant d'identifié l'id véhicule */
    QString codeConducteur; /**< QString contenant le code du conducteur */
    QString idService; /**< QString contenant le numéro de service en cours */
    QString idItineraire; /**< QString contenant le numéro d'itinéraire en cours */
    int enCourse; /**< int permettant de savoir si le conducteur est en course, pas en course ou si la course est terminée */
    int enService; /**< int permettant de savoir si le conducteur est en service ou non */
    int choixService; /**< int permettant de savoir quel est le service choisi par le conducteur */
    QStringList courseEnCours; /** StringList contenant le course en cours */
    QVector<QStringList> courses; /**< Vector de Stringlist contenant la liste des courses d'un service */
    BaseDeDonnees *bdd; /**< Pointeur bdd sur la base de donnée */
    QMutex mutex; /**< Mutex pour la gestion des temporisation */
    QWaitCondition waitCondition; /**< Pour la gestion des temporisation */

    /**
     * @brief Permet de faire des temporisations dans le thread
     *
     * @fn msleep valeur de la temporisation en millisecondes
     * @param sleepMS
     */
    void msleep(unsigned long sleepMS)
    {
        waitCondition.wait(&mutex, sleepMS);
    }
    /**
     * @brief Annule une temporisation en cours
     *
     * @fn annuler
     */
    void annuler()
    {
        waitCondition.wakeAll();
    }
    /**
     * @brief Permet de lire dans un fichier .ini
     *
     * @fn lireINI
     */
    void lireINI();
    /**
     * @brief Permet de charger le service d'un conducteur
     *
     * @fn chargerService
     * @return bool
     */
    bool chargerService();
    /**
     * @brief Permet de charger les courses d'un service
     *
     * @fn chargerCourses
     * @return bool
     */
    bool chargerCourses();
    /**
     * @brief Permet de charger la course à effectuer
     *
     * @fn demarrerCourse
     * @return bool
     */
    bool demarrerCourse();
    /**
     * @brief Met fin à la course en cours
     *
     * @fn terminerCourse
     * @return
     */
    void terminerCourse();

    /**
     * @brief Met fin au service en cours
     *
     * @fn terminerService
     * @return
     */
    void terminerService();

public:
/**
 * @brief Constructeur par défaut de la classe TGestionConducteur
 *
 * @fn TGestionConducteur
 */
    TGestionConducteur();
    /**
     * @brief Destructeur de la classe TGestionConducteur
     *
     * @fn ~TGestionConducteur
     */
    ~TGestionConducteur();

public slots:
    /**
     * @brief Le corp du thread
     *
     * @fn main
     */
    void main();
    /**
     * @brief Met fin à la boucle du thread
     *
     * @fn finir
     */
    void finir();
    /**
     * @brief Met fin au thread
     *
     * @fn terminer
     */
    void terminer();
    /**
     * @brief Slot déclenché par le signal envoyé par TInformationVoyageur
     *
     * @fn actualiserInformationsVoyageur
     * @param numLigne
     * @param destination
     * @param arretEnCours
     * @param prochainArret
     */
     void actualiserInformationsVoyageur(QString numLigne, QString destination, QString arretEnCours, QString prochainArret, int numeroArret);

signals:
    /**
     * @brief Signal actualisant les informations de la course en cours
     *
     * @fn actualiserCourse
     * @param numItineraire
     */
    void actualiserCourse(QString numItineraire);
    /**
     * @brief Signal indiquant qu'une course est finie pour le conducteur
     *
     * @fn courseFinie
     * @param idItineraire
     */
    void courseFinie(QString idItineraire);
    /**
     * @brief Signal indiquant la prise de service du conducteur
     *
     * @fn debutService
     * @param codeConducteur
     */
    void debutService(QString codeConducteur);
    /**
     * @brief Signal indiquant la fin de service du conducteur
     *
     * @fn finService
     * @param codeConducteur
     */
    void finService(QString codeConducteur);
};

#endif // TGESTIONCONDUCTEUR_H
