#ifndef TINFORMATIONVOYAGEUR_H
#define TINFORMATIONVOYAGEUR_H

#include <QtCore>
#include "tdiffusionmessages.h"
#include "hautparleur.h"

#define PI 3.1415

/// Etat du bus : PAS_EN_COURSE pour l'attribut enCourse
#define PAS_EN_COURSE   0
/// Etat du bus : EN_COURSE pour l'attribut enCourse
#define EN_COURSE       1
/// Etat du bus : COURSE_TERMINEE pour l'attribut enCourse
#define COURSE_TERMINEE 2

class BaseDeDonnees;

/**
 * @brief Classe qui informe les voyageurs et qui gère la detection des arrêts
 *
 * @class TInformationVoyageur tinformationvoyageur.h "tinformationvoyageur.h"
 *
 * @author Gabriel Destrée <gabriel.destree@gmail.com>
 */
class TInformationVoyageur : public QObject
{
    Q_OBJECT

private:
    TDiffusionMessages diffusionMessages; /**< Objet diffusionMessages de type TDiffusionMessages */
    QThread threadDiffusionMessages; /**< Objet threadDiffusionMessages de type QThread */
    HautParleur *hautParleur; /**< Objet hautParleur de type HautParleur */
    BaseDeDonnees *bdd; /**< Un pointeur sur un objet de type BaseDeDonnees */
    QVector<QStringList> itineraire; /**< Objet itineraire de type QVector<QStringList> */
    QString latitudeBus; /**< Un QString permettant de gérer la latitude GPS du bus */
    QString longitudeBus; /**< un QString permettant de gérer la longitude GPS du bus */
    int     nombreArrets; /**< Un int permettant de gérer le nombre d'arrêts sur une course */
    int     numeroArret; /**< Un int permettant de gérer le séquencement des arrêts sur une course */
    int     enCourse; /**< Un int qui permet de savoir si le bus est en course ou non */
    int     periode; /**< Un int correspondant au temps d'attente avant de redetecter si le bus est en course ou non */
    double  rayon; /**< Un double correspondant au rayon de detection des arrêts */
    bool    pret; /**< Un bool permettant de savoir si oui ou non le bus est pret (en fonction du conducteur) */
    bool    arret; /**< Un bool permmettant de savoir si le bus est sur un arret ou non */
    bool    fini; /**< Un bool permmettant de savoir lorsque le thread est terminé */

    QMutex mutex; /**< mutex pour la gestion des temporisations */
    QWaitCondition waitCondition; /**< pour la gestion des temporisations */
    /**
     * @brief Permet de faire des temporisations dans le thread
     *
     * @fn msleep
     * @param sleepMS
     */
    void msleep(unsigned long sleepMS)
    {
        waitCondition.wait(&mutex, sleepMS);
    }
    /**
     * @brief Annule une temporisation en cours
     *
     * @fn annuler
     */
    void annuler()
    {
        waitCondition.wakeAll();
    }

public:
    /**
     * @brief Constructeur de la classe TInformationVoyageur
     *
     * @fn TInformationVoyageur
     */
    TInformationVoyageur();
    /**
     * @brief Destruteur de la classe TInformationVoyageur
     *
     * @fn ~TInformationVoyageur
     */
    ~TInformationVoyageur();

    /**
     * @brief Lecture du fichier .ini et de ses paramètres
     *
     * @fn lireParametres
     */
    void lireParametres();
    /**
     * @brief Charge l'itineraire avec une requête SQL
     *
     * @fn chargerItineraire
     * @param numItineraire
     */
    void chargerItineraire(QString numItineraire);
    /**
     * @brief Permet de savoir si le bus est prêt
     *
     * @fn estPret
     * @return bool
     */
    bool estPret();
    /**
     * @brief Permet de savoir si le bus est sur le parcours
     *
     * @fn estEnCourse
     * @return int
     */
    int  estEnCourse();
    /**
     * @brief Permet de savoir si le bus a terminé sa course ou non
     *
     * @fn finirCourse
     * @return bool
     */
    bool finirCourse();
    /**
     * @brief Attente d'une periode avant réactualisation
     *
     * @fn attendrePeriode
     */
    void attendrePeriode();

    // Prototypes "déctecter les points d'arrêts"
    /**
     * @brief Methode principale de detection des arrêts
     *
     * @fn detecterArret
     */
    void detecterArret();
    /**
     * @brief Detecte lorsque le bus entre dans la zone d'arrêt
     *
     * @fn detecterDebutArret
     * @return bool
     */
    bool detecterDebutArret();

    /**
     * @brief Calcul la distance entre le bus et le rayon de detection
     *
     * @fn calculerDistance
     * @param latitudeBus
     * @param longitudeBus
     * @param latitudeArret
     * @param longitudeArret
     * @return double
     */
    double calculerDistance(QString latitudeBus, QString longitudeBus, QString latitudeArret, QString longitudeArret);

    /**
     * @brief Le bus est entré dans la zone d'arrêt
     *
     * @fn declencherDebutArret
     */
    void declencherDebutArret();
    /**
     * @brief Permet de savoir concretement si le bus est dans la zone d'arrêt
     *
     * @fn setZoneArret
     * @param etat
     */
    void setZoneArret(bool etat);
    /**
     * @brief Appelle des fonctions du haut parleur pour l'énonciation des arrêts
     *
     * @fn signalerArret
     */
    void signalerArret();
    /**
     * @brief Detecte lorsque le bus sort de la zone d'arrêt
     *
     * @fn detecterFinArret
     * @return bool
     */
    bool detecterFinArret();
    /**
     * @brief Le bus est sorti de la zone d'arrêt
     *
     * @fn declencherFinArret
     */
    void declencherFinArret();

public slots:
    /**
     * @brief Le corps du thread
     *
     * @fn main
     */
    void main();
    /**
     * @brief Met fin à la boucle du thread
     *
     * @fn finir
     */
    void finir();
    /**
     * @brief Termine le thread
     *
     * @fn terminer
     */
    void terminer();

    /**
     * @brief Actualisation de la position GPS du bus
     *
     * @fn actualiserLocalisation
     * @param latitude
     * @param longitude
     */
    void actualiserLocalisation(QString latitude, QString longitude);
    /**
     * @brief Actualisation d'une course utile à la detection des arrêts
     *
     * @fn actualiserCourse
     * @param numItineraire
     */
    void actualiserCourse(QString numItineraire);

signals:
    /**
     * @brief Signal qui envoie les informations déduites des méthodes de detection des arrêts
     *
     * @fn actualiserInformationsVoyageur
     * @param numLigne
     * @param destination
     * @param arretEnCours
     * @param prochainArret
     */
    void actualiserInformationsVoyageur(QString numLigne, QString destination, QString arretEnCours, QString prochainArret, int numeroArret);
};

#endif // TINFORMATIONVOYAGEUR_H
